@@ -1,7 +1,7 @@
 #ifndef DBDIMP_H
 #define DBDIMP_H
 /*
-   $Id: Ingres.sh,v 2.101 1997/09/12 06:26:47 ht000 Exp $
+   $Id: Ingres.sh,v 2.103 1997/09/25 11:46:36 ht000 Exp $
 
    Copyright (c) 1994,1995  Tim Bunce
    Copyright (c) 1996,1997  Henrik Tougaard (ht@datani.dk)
@@ -14,6 +14,7 @@
 #define NEED_DBIXS_VERSION 7
 
 #include <DBIXS.h>              /* installed by the DBI module  */
+#include <dbd_xsh.h>            /* ditto  */
 
 EXEC SQL INCLUDE SQLDA;
 EXEC SQL INCLUDE SQLCA;
@@ -40,6 +41,8 @@ struct imp_sth_st {
     int        st_num;      /* statement number */
     int        done_desc;   /* have we described this sth yet ?	*/
     IISQLDA    ph_sqlda;    /* descriptor for placeholders */
+    int        outerjoin;   /* true if this statement possibly is an
+                               outerjoin select */
     imp_fbh_t *fbh;	    /* array of imp_fbh_t structs	*/
 };
 
@@ -64,28 +67,32 @@ struct imp_fbh_st { 	    /* field buffer EXPERIMENTAL */
     } var_ptr;              /* buffer for data */
 };
 
+/* DBD::Ingres extensions */
+SV*     dbd_db_get_dbevent _((SV *dbh, imp_dbh_t *imp_dbh, SV *wait));
+#ifdef xxyyxxyyxxyyxx_ht
+#define dbd_db_get_dbevent      ing_db_get_dbevent
 
-void    dbd_init _((dbistate_t *dbistate));
-int	dbd_db_login _((SV *dbh, char* dbname, char* user, char* pass));
-int	dbd_db_do _((SV *dbh, char *statement, char *attribs, SV *params));
-int	dbd_db_rows _((SV *dbh, imp_dbh_t *imp_dbh));
-int	dbd_db_commit _((SV *dbh));
-int	dbd_db_rollback _((SV * dbh));
-int	dbd_db_disconnect _((SV * dbh));
-void	dbd_db_destroy _((SV * dbh));
-int	dbd_db_STORE_attrib _((SV *dbh, SV *keysv, SV* valuesv));
-SV*	dbd_db_FETCH_attrib _((SV *dbh, SV *keysv));
-int	dbd_st_prepare _((SV * sth, char *statement, SV* attribs));
-int     dbd_describe _((SV *h, imp_sth_t *imp_sth));
-int     dbd_bind_ph  _((SV *sth, imp_sth_t *imp_sth,
-                SV *param, SV *value, IV sql_type, SV *attribs,
-				int is_inout, IV maxlen));
-int	dbd_st_execute _((SV *sth));
-AV*	dbd_st_fetch _((SV *sth));
-int	dbd_st_rows _((SV *sth, imp_sth_t *imp_sth));
-int	dbd_st_finish _((SV *sth));
-void	dbd_st_destroy _((SV *sth));
-int	dbd_st_STORE_attrib _((SV *dbh, SV *keysv, SV* valuesv));
-SV*	dbd_st_FETCH_attrib _((SV *dbh, SV *keysv));
+/* These defines avoid name clashes for multiple statically linked DBD's	*/
+#define dbd_init		ing_init
+#define dbd_db_login		ing_db_login
+#define dbd_db_do		ing_db_do
+#define dbd_db_commit		ing_db_commit
+#define dbd_db_rollback		ing_db_rollback
+#define dbd_db_disconnect	ing_db_disconnect
+#define dbd_db_destroy		ing_db_destroy
+#define dbd_db_STORE_attrib	ing_db_STORE_attrib
+#define dbd_db_FETCH_attrib	ing_db_FETCH_attrib
+#define dbd_st_prepare		ing_st_prepare
+#define dbd_st_rows		ing_st_rows
+#define dbd_st_execute		ing_st_execute
+#define dbd_st_fetch		ing_st_fetch
+#define dbd_st_finish		ing_st_finish
+#define dbd_st_destroy		ing_st_destroy
+#define dbd_st_blob_read	ing_st_blob_read
+#define dbd_st_STORE_attrib	ing_st_STORE_attrib
+#define dbd_st_FETCH_attrib	ing_st_FETCH_attrib
+#define dbd_describe		ing_describe
+#define dbd_bind_ph		ing_bind_ph
 /* end */
 #endif
+#endif
